def create_profile(self, body: dict = None):
    """
    创建
    :return:
    """
    params = {
        "Body": body,
    }
    return self.send_request('POST', '/profile/create', params)


def update_profile(self, profile_id: str = None, body: dict = None):
    """
    更新配置

    :param profile_id: 配置ID
    :param body: 账号信息
    :return:
    """
    params = {
        'profileId': profile_id,
        'Body': body,
    }
    return self.send_request('POST', '/profile/update', params)


def get_profile_detail(self, profile_id: str = None):
    """
    获取配置详情
    :param profile_id: 配置ID
    :return:

    """
    params = {
        'profileId': profile_id,
    }
    return self.query('/profile/detail', params)


def share_profile(self, profile_id: str = None, account: str = ''):
    """
    分享配置
    :param profile_id: 浏览器配置文件ID（多条以,拼接
    :param account: 用户名
    :return:
    """
    params = {
        'profileId': profile_id,
        'account': account,
    }
    return self.query('/profile/share', params)


def cache_share_profile(self, profile_id: str = None, account: str = ''):
    """
    分享配置
    :param profile_id: 浏览器配置文件ID（多条以,拼接
    :param account: 用户名
    :return:
    """
    params = {
        'profileId': profile_id,
        'account': account,
    }
    return self.query('/profile/cancelShare', params)


def transfer_profile(self, profile_id: str = None, account: str = ''):
    """
    转移配置
    :param profile_id: 浏览器配置文件ID（多条以,拼接
    :param account: 用户名
    :return:
    """
    params = {
        'profileId': profile_id,
        'account': account,
    }
    return self.query('/profile/transferOwnership', params)


def release_profile(self, profile_id: str = None):
    """
    释放配置
    :param profile_id: 浏览器配置文件ID（多条以,拼接
    :return:
    """
    params = {
        'profileId': profile_id,
    }
    return self.query('/profile/release', params)


def remove_profile(self, profile_id: str = None):
    """
    删除
    :param profile_id: 浏览器配置文件ID（多条以,拼接）
    :return:
    """
    params = {
        'profileId': profile_id,
    }
    return self.query('/profile/remove', params)


def get_profile_list(self, search: str = None, tag: str = None, format: str = None, page: str = None, limit: str = 10):
    """
    获取配置列表
    :param search: 搜索关键字
    :param tag: 分组
    :param format: “1”: 将返回值unix时间戳格式化输出
    :param page: 分页页码
    :param limit: 每页显示多少条记录, 默认500, 最大1000
    :return:
    """
    params = {
        'search': search,
        'tag': tag,
        'format': format,
        'page': page,
        'limit': limit,
    }
    return self.query('/profile/list', params)



def start(self, profile_id: str = None, skiplock: bool = False, block: bool = False):
    """
    启动配置
    :param profile_id: 浏览器配置文件ID
    :param skiplock: 跳过会话锁启动浏览器
    :param block: 接口将持续阻塞中, 直到浏览器进程创建成功后才返回结果 (for V1.3.7.7)
    :return:
    """
    params = {
        'profileId': profile_id,
        'skiplock': skiplock,
        'block': block,
    }
    return self.query('/profile/start', params)


def stop(self, profile_id: str = None, force: bool = False, block: bool = False):
    """
    启动配置
    :param profile_id: 浏览器配置文件ID
    :param force: 强制关闭浏览器 [移动仿真]建议为 true, 以免无法关闭浏览器
    :param block: 接口持续阻塞中, 直到浏览器进程结束 (for V1.3.7.7)
    :return:
    """
    params = {
        'profileId': profile_id,
        'force': force,
        'block': block,
    }
    return self.query('/profile/stop', params)


def refresh(self, profile_id: str = None):
    """
    刷新配置
    :param profile_id: 浏览器配置文件ID
    :return:
    """
    params = {
        'profileId': profile_id,
    }
    return self.query('/profile/refresh', params)


def open_url(self, profile_id: str = None, url: str = None):
    """
    打开网址
    :param profile_id: 浏览器配置文件ID
    :param url: 网址
    :return:
    """
    params = {
        'profileId': profile_id,
        'url': url,
    }
    return self.query('/profile/openurl', params)


def new_tab(self, profile_id: str = None, url: str = None):
    """
    新建标签页
    :param profile_id: 浏览器配置文件ID
    :param url: 网址
    :return:
    """
    params = {
        'profileId': profile_id,
        'url': url,
    }
    return self.query('/profile/newtab', params)


def source(self, profile_id: str = None):
    """
    获取配置源码
    :param profile_id: 浏览器配置文件ID
    :return:
    """
    params = {
        'profileId': profile_id,
    }
    return self.query('/profile/source', params)


def find_element(self, profile_id: str = None, function: str = None, args: str = None, click: bool = False,
                 index: int = 0):
    """
    查找元素
    :param profile_id: 浏览器配置文件ID
    :param function: 查找元素的函数名
    :param args: 参数为 function函数的传参，如：findElementByID(args)
    :param click: 找到指定元素后是否点击操作 true:点击 false:不点击
    :param index: 元素有多个时，指定点击第几个元素。从0开始，-1 为随机元素其中一个，配合 click = true 时使用
    :return:

    """
    params = {
        'profileId': profile_id,
        'function': function,
        'args': args,
        'click': click,
        'index': index,
    }
    return self.query('/profile/findElement', params)


def get_attribute(self, profile_id: str = None, args: str = None):
    """
    获取元素属性
    :param profile_id: 浏览器配置文件ID
    :param args: 一般是通过findElement查找元素后得到的Element_ID结合成的JSON串。如：[{“ELEMENT”: “95365abe-6746-4703-8cfd-6316580f5289”, “element-6066-11e4-a52e-4f735466cecf”: “95365abe-6746-4703-8cfd-6316580f5289”}, “value”] 这个就是取 95365abe-6746-4703-8cfd-6316580f5289 元素的 value 值。
    :return:
    """
    params = {
        'profileId': profile_id,
        'args': args,
    }
    return self.query('/profile/getAttribute', params)


def send_keys(self, profile_id: str = None, element_id: str = None, value: str = None):
    """
    发送文字到网页中

    :param profile_id: 浏览器配置文件ID
    :param element_id: 通过findElement查找元素后得到的elementId
    :param value: 向这个网页控件中发送字符串（ Enter Escape Tab vkLeft vkUp vkRight vkDown vkDelete vkEnd vkHome vkBack）
    :return:
    """
    params = {
        'profileId': profile_id,
        'elementId': element_id,
        'value': value,
    }
    return self.query('/profile/sendKeys', params)


def active(self, profile_id: str = None, active: bool = True):
    """
    检查配置文件是否正在运行

    :param profile_id: 浏览器配置文件ID
    :param active: true: 激活当前配置浏览器到最前面 false: 默认
    :return:
    """
    params = {
        'profileId': profile_id,
        'active': active,
    }
    return self.query('/profile/active', params)


def random_profile(self, platform: str = None, lang_hdr: str = None, accept_language: str = None,
                   time_zone: str = None):
    """
    随机获取配置信息

    :param platform: 平台类型 Windows Linux Macintosh Android iPhone Chrome Firefox Edge
    :param lang_hdr: 语言 en-US
    :param accept_language: 语言 en-US,en;q=0.9
    :param time_zone: 时区 America/New_York
    :return:
    """
    params = {
        'platform': platform,
        'langHdr': lang_hdr,
        'acceptLanguage': accept_language,
        'timeZone': time_zone,
    }
    return self.query('/profile/randomProfile', params)


def delete(self, profile_id: str = None):
    """
    删除一个配置文件本地临时文件夹

    :param profile_id: 浏览器配置文件ID
    :return:
    """
    params = {
        'profileId': profile_id,
    }
    return self.query('/profile/delete', params)


def cookie_export(self, profile_id: str = None):
    """
    从配置文件导出cookie

    :param profile_id: 浏览器配置文件ID
    :return:
    """
    params = {
        'profileId': profile_id,
    }
    return self.query('/profile/cookies/export/webext', params)


def cookie_import(self, profile_id: str = None, body: str = None):
    """
    将cookie导入配置文件浏览器

    :param profile_id: 浏览器配置文件ID
    :param body: cookie json (need base64 encode )
    :return:
    """
    params = {
        'profileId': profile_id,
        'body': body,
    }
    return self.send_request('POST', '/profile/cookies/import/webext', params)


def execute_script(self, profile_id: str = None, body: str = None):
    """
    执行js脚本

    :param profile_id: 浏览器配置文件ID
    :param body: js code (need base64 encode )
    :return:
    """
    params = {
        'profileId': profile_id,
        'body': body,
    }
    return self.send_request('POST', '/profile/ExecuteScript', params)


def screen_shot(self, profile_id: str = None, body: str = None):
    """
    截图

    :param profile_id: 浏览器配置文件ID
    :param body: 图片的本机绝对路径地址 (need base64 encode )
    :return:
    """
    params = {
        'profileId': profile_id,
        'body': body,
    }
    return self.send_request('POST', '/profile/ScreenShot', params)


def get_cookies(self, profile_id: str = None, all: int = 0):
    """
    获取配置文件的cookie

    :param profile_id: 浏览器配置文件ID
    :param all: 0: No:0 Yes:1
    :return:
    """
    params = {
        'profileId': profile_id,
        'all': all,
    }
    return self.send_request('POST', '/profile/getCookies', params)


def ready_state(self, profile_id: str = None):
    """
    询问当前页面也没有加载完成(

    :param profile_id: 浏览器配置文件ID
    :return:
    """
    params = {
        'profileId': profile_id,
    }
    return self.send_request('POST', '/profile/readyState', params)


def switch_frame(self, profile_id: str = None, args: str = None):
    """
    切换到指定的frame

    :param profile_id: 浏览器配置文件ID
    :param args:FrameName（支持base64编码）
    :return:
    """
    params = {
        'profileId': profile_id,
        'args': args,
    }
    return self.query('/profile/SwitchToFrame', params)


def switch_tab(self, profile_id: str = None, id: str = None):
    """
    切换到指定的tab

    :param profile_id: 浏览器配置文件ID
    :param id: /profile/start 返回的地址和端口（value）拼接/json（例如：http://127.0.0.1:18500/json）返回的tab列表（json）中的id 字段
    :return:
    """
    params = {
        'profileId': profile_id,
        'id': id,
    }
    return self.query('/profile/bringToFront', params)


def close_tab(self, profile_id: str = None, id: str = None):
    """
    关闭指定的tab

    :param profile_id: 浏览器配置文件ID
    :param id: /profile/start 返回的地址和端口（value）拼接/json（例如：http://
    :return:
    """
    params = {
        'profileId': profile_id,
        'id': id,
    }
    return self.query('/profile/page/close', params)


def password_manager_leak_detection(self, profile_id: str = None, enable: bool = False):
    """
    密码泄露检测

    :param profile_id: 浏览器配置文件ID
    :param body: 关闭:false 开启:true 默认关闭
    :return:
    """
    params = {
        'profileId': profile_id,
        'enable': enable,
    }
    return self.send_request('POST', '/profile/browser/password_manager_leak_detection', params)


def proxy_test(self, proxy_type: str = None, proxy_server: str = None, proxy_port: str = None, **kwargs):
    """
    代理测试

    :param proxy_type: 代理类型 http socks5
    :param proxy_server: 代理服务器地址
    :param proxy_port: 代理服务器端口
    :kwarg proxy_username: 代理服务器用户名
    :kwarg proxy_password: 代理服务器密码
    :kwarg timeout: 超时时间
    :kwarg urlindex: 代理测试网址选项 0 = lumtest 1 = vmlogin 2 = ip-api.com
    :return:
    """
    params = {
        'proxyType': proxy_type,
        'proxyServer': proxy_server,
        'proxyPort': proxy_port,
        'proxyUsername': kwargs.get('proxy_username', None),
        'proxyPassword': kwargs.get('proxy_password', None),
        'timeout': kwargs.get('timeout', 10),
        'urlindex': kwargs.get('urlindex', 0),
    }
    return self.query('/proxy/test', params)


def restart(self):
    """
    重启服务

    :return:
    """
    return self.query('/client/restart')



def create_group(self, name: str = None, orderby: int = 0):
    """
    创建分组
    :param name: 分组名称
    :param orderby: 排序

    :return:
    """
    params = {
        'name': name,
        'orderby': orderby,
    }
    return self.query('/tag/create', params)


def remove_group(self, tag_id: str = None):
    """
    删除分组
    :param tag_id: 分组ID

    :return:
    """
    params = {
        'tagId': tag_id,
    }
    return self.query('/tag/remove', params)


def rename_group(self, name: str = None, tag_id: str = None, orderby: int = 0):
    """
    重命名分组
    :param name: 分组名称
    :param tag_id: 分组ID
    :param orderby: 排序
    """
    params = {
        'name': name,
        'tagId': tag_id,
        'orderby': orderby,
    }
    return self.query('/tag/rename', params)


def get_group_list(self):
    """
    获取分组列表
    :return:
    """
    return self.query('/tag/list')


def profile_bind_group(self, tag_id: str = None, profile_id: str = None):
    """
    绑定分组
    :param tag_id: 分组ID
    :param profile_id: 账号ID
    :return:
    """
    params = {
        'tagId': tag_id,
        'profileId': profile_id,
    }
    return self.query('/tag/profile/add', params)


def profile_unbind_group(self, tag_id: str = None, profile_id: str = None):
    """
    解绑分组
    :param tag_id: 分组ID
    :param profile_id: 账号ID
    :return:
    """
    params = {
        'tagId': tag_id,
        'profileId': profile_id,
    }
    return self.query('/tag/profile/remove', params)



from VmLogin.api import API


class Local(API):
    def __init__(self, **kwargs):
        if "base_url" not in kwargs:
            kwargs["base_url"] = "http://127.0.0.1:35000/api/v1"
        super().__init__('', **kwargs)

    from VmLogin.local.browser import start
    from VmLogin.local.browser import stop
    from VmLogin.local.browser import refresh
    from VmLogin.local.browser import open_url
    from VmLogin.local.browser import new_tab
    from VmLogin.local.browser import source
    from VmLogin.local.browser import find_element
    from VmLogin.local.browser import get_attribute
    from VmLogin.local.browser import send_keys
    from VmLogin.local.browser import active
    from VmLogin.local.browser import random_profile
    from VmLogin.local.browser import delete
    from VmLogin.local.browser import cookie_export
    from VmLogin.local.browser import cookie_import
    from VmLogin.local.browser import execute_script
    from VmLogin.local.browser import screen_shot
    from VmLogin.local.browser import get_cookies
    from VmLogin.local.browser import ready_state
    from VmLogin.local.browser import switch_frame
    from VmLogin.local.browser import switch_tab
    from VmLogin.local.browser import close_tab
    from VmLogin.local.browser import password_manager_leak_detection
    from VmLogin.local.browser import proxy_test
    from VmLogin.local.browser import restart



import requests
import json
from json import JSONDecodeError
from VmLogin.error import ClientError, ServerError


class API(object):
    def __init__(self, token=None, base_url=None):
        self.token = token
        self.base_url = base_url
        self.session = requests.Session()

    def query(self, url_path, payload=None):
        return self.send_request("GET", url_path, payload=payload)

    def _dispatch_request(self, http_method):
        return {
            "GET": self.session.get,
            "DELETE": self.session.delete,
            "PUT": self.session.put,
            "POST": self.session.post,
        }.get(http_method, "GET")

    def _handle_exception(self, response):
        status_code = response.status_code
        if status_code < 400:
            return
        if 400 <= status_code < 500:
            try:
                err = json.loads(response.text)
            except JSONDecodeError:
                raise ClientError(
                    status_code, None, response.text, None, response.headers
                )
            error_data = None
            if "data" in err:
                error_data = err["data"]
            raise ClientError(
                status_code, err["code"], err["msg"], response.headers, error_data
            )
        raise ServerError(status_code, response.text)

    def send_request(self, http_method, url_path, payload=None):
        if payload is None:
            payload = {}
        url = self.base_url + url_path
        payload["token"] = self.token
        if http_method == "GET":
            params = {
                "url": url,
                "params": payload,
            }
        else:
            params = {
                "url": url,
                "data": json.dumps(payload),
            }
        response = self._dispatch_request(http_method)(**params)
        self._handle_exception(response)
        try:
            data = response.json()
        except ValueError:
            data = response.text
        result = {}

        if len(result) != 0:
            result["data"] = data
            return result

        return data



from VmLogin.api import API


class Online(API):
    def __init__(self, token: str = '', **kwargs):
        if "base_url" not in kwargs:
            kwargs["base_url"] = "https://api.vmlogin.com/v1"
        super().__init__(token, **kwargs)

    from VmLogin.online.groups import create_group
    from VmLogin.online.groups import remove_group
    from VmLogin.online.groups import rename_group
    from VmLogin.online.groups import get_group_list
    from VmLogin.online.groups import profile_bind_group
    from VmLogin.online.groups import profile_unbind_group

    from VmLogin.online.profiles import create_profile
    from VmLogin.online.profiles import remove_profile
    from VmLogin.online.profiles import get_profile_detail
    from VmLogin.online.profiles import update_profile
    from VmLogin.online.profiles import share_profile
    from VmLogin.online.profiles import cache_share_profile
    from VmLogin.online.profiles import transfer_profile
    from VmLogin.online.profiles import release_profile
    from VmLogin.online.profiles import get_profile_list
